/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 11 fevrier 2007
   derniere modification le 19/10/2007 */

#include <SCE/SCEMinimal.h>

#include <SCE/utils/SCEString.h>
#include <SCE/core/SCECSupport.h>

#include <SCE/core/SCECShader.h>


/* variables internes : target du dernier shader ASM binde */
static SCEenum last_target = 0;


/* Cg */
#ifdef _SCE_USE_CG
/* constantes internes */
/* macros qui serviront a CgManager */
#define SCE_INIT 1
#define SCE_QUIT 2
#define SCE_GET_STATE 3


/* NOTE: truc douteux.. :/ ? */
/* a, b, c.. -> 3
   a, b, c, d, e, f, g.. -> 7
   Cg */
#define SCE_CG_ERROR (SCE_NUM_ERRORS+37)

/* variables statiques necessaires a l'utilisation de Cg */
static CGcontext context;
static CGprofile vs_profile;
static CGprofile ps_profile;


/* fonction callback en cas d'erreur Cg */
static void SCE_CCgOnError (void)
{
    Logger_Log (SCE_CG_ERROR);
    Logger_LogMsg ("a Cg error was occured :\n- %s\n- %s",
                   cgGetErrorString(cgGetError()), cgGetLastListing(context));
}


/* fonction de manipulation du runtime Cg */
static int SCE_CCgManager (int action)
{
    static int is_init = SCE_FALSE;

    SCE_btstart ();
    switch (action)
    {
    case SCE_INIT:
        if(!is_init)
        {
            /* envoie de la fonction callback pour la gestion des erreurs */
            cgSetErrorCallback (SCE_CCgOnError);

            /* creation du contexte */
            context = cgCreateContext ();
            if (Logger_HaveError () && Logger_GetCode () == SCE_CG_ERROR)
            {
                Logger_LogSrc ();
                is_init = SCE_ERROR;
                break; /* on sort... */
            }

            /* chargement des 'profiles' */
            vs_profile = cgGLGetLatestProfile (CG_GL_VERTEX);
            ps_profile = cgGLGetLatestProfile (CG_GL_FRAGMENT);
            cgGLSetOptimalOptions (vs_profile);
            cgGLSetOptimalOptions (ps_profile);

            is_init = 1;
        }
        break;

    case SCE_QUIT:
        if (is_init)
        {
            if (context)
                cgDestroyContext (context);

            context = NULL; /* CGcontext n'est qu'un typedef struct* ... */
            vs_profile = CG_PROFILE_UNKNOWN;
            ps_profile = CG_PROFILE_UNKNOWN;

            is_init = 0;
        }
        break;

    /* rien a faire de particulier pour get state */
    case SCE_GET_STATE:
    }

    SCE_btend ();
    return is_init;
}
#endif /* _SCE_USE_CG */

/* renvoie un booleen qui vaut true si type est un shader de pixel */
static int SCE_CIsPixelShader (SCEenum type)
{
    return (type == SCE_PIXEL_SHADER);
}



/* revise le 18/10/2007 */
int SCE_CShaderInit (int use_cg)
{
    SCE_btstart ();
    #ifdef _SCE_USE_CG
    if (use_cg)
    {
        if (!SCE_CCgManager (SCE_INIT))
        {
            Logger_LogSrc ();
            SCE_btend ();
            return SCE_ERROR;
        }
    }
    #else /* pour eviter un warning : unused parameter 'use_cg' */
    use_cg = 0;
    #endif

    SCE_btend ();
    return SCE_OK;
}

void SCE_CShaderQuit (void)
{
    #ifdef _SCE_USE_CG
    SCE_CCgManager (SCE_QUIT);
    #endif
}



#ifdef _SCE_USE_CG
CGprofile SCE_CGetCgProfile(SCEenum type)
{
    return (type == SCE_PIXEL_SHADER) ? ps_profile : vs_profile;
}
#endif

/* *** */

SCE_CShaderGLSL* SCE_CCreateShaderGLSL(SCEenum type)
{
    SCE_CShaderGLSL *shader = NULL;

    SCE_btstart ();
    shader = SCE_malloc (sizeof *shader);
    if (!shader)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return NULL;
    }

    shader->data = NULL;
    shader->compiled = SCE_FALSE;
    shader->is_pixelshader = SCE_CIsPixelShader (type);
    shader->type = (shader->is_pixelshader) ?
                    GL_FRAGMENT_SHADER : GL_VERTEX_SHADER;

    shader->id = glCreateShader (shader->type);
    if (shader->id == 0)
    {
        /* une erreur est survenue lors de la creation du shader */
        Logger_Log (SCE_ERROR);
        Logger_LogMsg ("I can't create a shader, what's the fuck ?");
        SCE_free (shader);
        SCE_btend ();
        return NULL;
    }

    SCE_btend ();
    return shader;
}

SCE_CShaderASM* SCE_CCreateShaderASM (SCEenum type)
{
    SCE_CShaderASM *shader = NULL;

    SCE_btstart ();
    shader = SCE_malloc (sizeof *shader);
    if (!shader)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return NULL;
    }

    shader->data = NULL;
    shader->compiled = SCE_FALSE;
    shader->is_pixelshader = SCE_CIsPixelShader(type);
    shader->type = (shader->is_pixelshader) ?
                    GL_FRAGMENT_PROGRAM_ARB : GL_VERTEX_PROGRAM_ARB;

    glGenProgramsARB (1, &shader->id);

    SCE_btend ();
    return shader;
}

#ifdef _SCE_USE_CG
SCE_CShaderCG* SCE_CCreateShaderCG (SCEenum type)
{
    SCE_CShaderCG *shader = NULL;

    SCE_btstart ();
    shader = SCE_malloc (sizeof *shader);
    if (!shader)
    {
        Logger_LogSrc();
        SCE_btend ();
        return NULL;
    }

    shader->id = NULL;
    shader->args = NULL;
    shader->data = NULL;
    shader->compiled = SCE_FALSE;
    shader->is_pixelshader = SCE_CIsPixelShader (type);
    shader->type = SCE_CGetCgProfile (type);

    SCE_btend ();
    return shader;
}
#endif

/* *** */

/* revise le 18/10/2007 */
void SCE_CDeleteShaderGLSL (SCE_CShaderGLSL *shader)
{
    SCE_btstart ();
    if (shader)
    {
        if (glIsShader (shader->id))
            glDeleteShader (shader->id);
        SCE_free (shader->data);
        SCE_free (shader);
    }
    SCE_btend ();
}

/* revise le 18/10/2007 */
void SCE_CDeleteShaderASM (SCE_CShaderASM *shader)
{
    SCE_btstart ();
    if (shader)
    {
        glDeleteProgramsARB (1, &shader->id);
        SCE_free (shader->data);
        SCE_free (shader);
    }
    SCE_btend ();
}

#ifdef _SCE_USE_CG
/* revise le 18/10/2007 */
void SCE_CDeleteShaderCG (SCE_CShaderCG *shader)
{
    SCE_btstart ();
    if (shader)
    {
        if (shader->id)
            cgDestroyProgram (shader->id);
        if (shader->args)
        {
            usigned int i;
            for (i=0; shader->args[i]; i++)
                SCE_free (shader->args[i]);
            SCE_free (shader->args);
        }
        SCE_free (shader->data);
        SCE_free (shader);
    }
    SCE_btend ();
}
#endif

/* *** */

/* revise le 18/10/2007 */
void SCE_CSetShaderGLSLSource (SCE_CShaderGLSL *shader, char *src)
{
    SCE_free (shader->data);
    shader->data = src;
}
/* ajoute le 18/10/2007 */
int SCE_CSetShaderGLSLSourceDup (SCE_CShaderGLSL *shader, char *src)
{
    char *new = NULL;

    SCE_btstart ();
    new = SCE_String_Dup (src);
    if (!new)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }
    SCE_CSetShaderGLSLSource (shader, new);
    SCE_btend ();
    return SCE_OK;
}

/* revise le 18/10/2007 */
void SCE_CSetShaderASMSource (SCE_CShaderASM *shader, char *src)
{
    SCE_free (shader->data);
    shader->data = src;
}
/* ajoute le 18/10/2007 */
int SCE_CSetShaderASMSourceDup (SCE_CShaderASM *shader, char *src)
{
    char *new = NULL;

    SCE_btstart ();
    new = SCE_String_Dup (src);
    if (!new)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }
    SCE_CSetShaderASMSource (shader, new);
    SCE_btend ();
    return SCE_OK;
}

#ifdef _SCE_USE_CG
/* revise le 18/10/2007 */
void SCE_CSetShaderCGSource (SCE_CShaderCG *shader, char *src)
{
    SCE_free (shader->data);
    shader->data = src;
}
/* ajoute le 18/10/2007 */
int SCE_CSetShaderCGSourceDup (SCE_CShaderCG *shader, char *src)
{
    char *new = NULL;

    SCE_btstart ();
    new = SCE_String_Dup (src);
    if (!new)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }
    SCE_CSetShaderCGSource (shader, new);
    SCE_btend ();
    return SCE_OK;
}

/* revise le 18/10/2007 */
void SCE_CSetShaderCGArgs (SCE_CShaderCG *shader, char **args)
{
    SCE_btstart ();
    if (shader->args)
    {
        usigned int i;
        for (i=0; shader->args[i]; i++)
            SCE_free (shader->args[i]);
        SCE_free (shader->args);
    }
    shader->args = args;
    SCE_btend ();
}
/* ajoute le 18/10/2007 */
int SCE_CSetShaderCGArgsDup (SCE_CShaderCG *shader, char **args)
{
    size_t s = 0, i;
    char **new;

    SCE_btstart ();
    while (args[s])
        s++;
    new = SCE_malloc (s * sizeof *new + 1);
    if (!new)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }
    for (i=0; i<s; i++)
    {
        if (!(new[i] = SCE_String_Dup (args[i])))
        {
            Logger_LogSrc ();
            SCE_btend ();
            return SCE_ERROR;
        }
    }
    new[s] = NULL;
    SCE_CSetShaderCGArgs (shader, new);
    SCE_btend ();
    return SCE_OK;
}
#endif

/* *** */


/* ajoute le 18/10/2007 */
/* fonctions generique des erreurs de construction */
static int SCE_CCantRecompile (void)
{
    Logger_Log (SCE_INVALID_OPERATION);
    Logger_LogMsg ("you can't re-compile a shader");
    return SCE_ERROR;
}
static int SCE_CNeedCode (void)
{
    Logger_Log (SCE_INVALID_OPERATION);
    Logger_LogMsg ("you can't compile a shader without source code");
    return SCE_ERROR;
}

int SCE_CBuildShaderGLSL (SCE_CShaderGLSL *shader)
{
    int compile_status = GL_TRUE;
    int loginfo_size = 0;
    char *loginfo = NULL;  /* journal de compilation (info log) */

    SCE_btstart ();
    /* NOTE: et pourquoi on pourrait pas recompiler un shader ?? */
    if (shader->compiled)
    {
        SCE_btend ();
        return SCE_CCantRecompile ();
    }

    if (!shader->data)
    {
        SCE_btend ();
        return SCE_CNeedCode ();
    }

    /* cast: fucking hack */
    glShaderSource (shader->id, 1, (const GLchar**)&shader->data, NULL);
    glCompileShader (shader->id);

    /* verification du succes de la compilation */
    glGetShaderiv (shader->id, GL_COMPILE_STATUS, &compile_status);
    if (compile_status != GL_TRUE)
    {
        /* erreur a la compilation
           recuperation du log d'erreur */

        Logger_Log (SCE_INVALID_OPERATION);

        glGetShaderiv (shader->id, GL_INFO_LOG_LENGTH, &loginfo_size);
        loginfo = SCE_malloc (loginfo_size + 1);
        if (!loginfo)
        {
            Logger_LogSrc ();
            SCE_btend ();
            return SCE_ERROR;
        }

        memset (loginfo, '\0', loginfo_size + 1);
        glGetShaderInfoLog (shader->id, loginfo_size, &loginfo_size, loginfo);

        Logger_LogMsg ("error while compiling GLSL %s shader :\n%s",
                       shader->is_pixelshader ? "pixel":"vertex", loginfo);

        SCE_free (loginfo);
        SCE_btend ();
        return SCE_ERROR;
    }

    shader->compiled = SCE_TRUE;
    SCE_btend ();
    return SCE_OK;
}

int SCE_CBuildShaderASM (SCE_CShaderASM *shader)
{
    /*int msg_size = 0;*/
    /* message pour les shaders ASM, qui renvoient l'erreur par glGetString */
    const char *msg;

    SCE_btstart ();
    if (shader->compiled)
    {
        SCE_btend ();
        return SCE_CCantRecompile ();
    }

    if (!shader->data)
    {
        SCE_btend ();
        return SCE_CNeedCode ();
    }

    glBindProgramARB (shader->type, shader->id);

    /* envoie du code source */
    /* NOTE: si opengl supporte, prevoir une boucle pour envoyer
             l'integrite du tableau des chaines,
             la on envoie que la premiere chaine de caracteres */
    glProgramStringARB (shader->type, GL_PROGRAM_FORMAT_ASCII_ARB,
                        strlen(shader->data), shader->data);

    if (glGetError () == GL_INVALID_OPERATION)
    {
        /* une erreur est survenue lors de l'envoie des donnees */

        Logger_Log (SCE_INVALID_OPERATION);

        msg = (char*)glGetString (GL_PROGRAM_ERROR_STRING_ARB);
        /* ici, msg_size sert de repere pour siter
           ou s'est produite l'erreur */
        /*glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &msg_size);*/

        Logger_LogMsg ("error while compiling ASM %s program :\n%s",
                       shader->is_pixelshader ? "fragment":"vertex", msg);

        glBindProgramARB (shader->type, 0);
        SCE_btend ();
        return SCE_ERROR;
    }

    glBindProgramARB (shader->type, 0);
    shader->compiled = SCE_TRUE;

    SCE_btend ();
    return SCE_OK;
}

#ifdef _SCE_USE_CG
int SCE_CBuildShaderCG (SCE_CShaderCG *shader)
{
    SCE_btstart ();
    /* on verifie si Cg a ete initialise */
    if (!SCE_CCgManager (SCE_GET_STATE))
    {
        /* le contexte Cg n'a pas ete ou a ete detrui, arret */
        Logger_Log (SCE_INVALID_OPERATION);
        Logger_LogMsg ("you can't build a Cg shader if you have"
                       " not initialized the shaders manager");
        SCE_btend ();
        return SCE_ERROR;
    }

    if (shader->compiled)
    {
        SCE_btend ();
        return SCE_CCantRecompile ();
    }

    if (!shader->data)
    {
        SCE_btend ();
        return SCE_CNeedCode ();
    }

    #define SCE_CG_VERIF()\
    if (Logger_HaveError () && Logger_GetCode () == SCE_CG_ERROR)\
    {\
        Logger_LogSrc ();\
        SCE_btend ();\
        return SCE_ERROR;\
    }

    shader->id = cgCreateProgram (context, CG_SOURCE, shader->data,
                                  shader->type, "main",
                                  (const char**)shader->args);
    SCE_CG_VERIF ()

    /* NOTE: faudra que je voye si c'est bien necessaire tout ca */
    /* compilation du program */
    if (!cgIsProgramCompiled (shader->id))
    {
        /* program non compile -> alors on le fait */
        cgCompileProgram (shader->id);
    }

    /* et on fait CA pour 'charger' le shader ?? */
    cgGLLoadProgram (shader->id);
    SCE_CG_VERIF ()

    #undef SCE_CG_VERIF

    shader->compiled = SCE_TRUE;

    SCE_btend ();
    return SCE_OK;
}
#endif

/* *** */

void SCE_CUseShaderASM (SCE_CShaderASM *shader)
{
    static int is_ps = -1;

    if (shader)
    {
        glEnable (shader->type);
        glBindProgramARB (shader->type, shader->id);
        
        is_ps = (is_ps == SCE_TRUE) ? SCE_TRUE : shader->is_pixelshader;
        last_target = shader->type;
    }
    else if (is_ps == SCE_TRUE)
    {
        glBindProgramARB (GL_FRAGMENT_PROGRAM_ARB, 0);
        glDisable (GL_FRAGMENT_PROGRAM_ARB);
        is_ps = SCE_FALSE;
        last_target = GL_VERTEX_PROGRAM_ARB;
    }
    else if (is_ps == SCE_FALSE)
    {
        glBindProgramARB (GL_VERTEX_PROGRAM_ARB, 0);
        glDisable (GL_VERTEX_PROGRAM_ARB);
        is_ps = -1;
        last_target = 0;
    }
}

#ifdef _SCE_USE_CG
void SCE_CUseShaderCG (SCE_CShaderCG *shader)
{
    static int is_ps = -1;

    if (shader)
    {
        cgGLBindProgram (shader->id);
        cgGLEnableProfile (shader->type);
        
        is_ps = (is_ps == SCE_TRUE) ? SCE_TRUE : shader->is_pixelshader;
    }
    else if (is_ps == SCE_TRUE)
    {
        cgGLDisableProfile (ps_profile);
        cgGLUnbindProgram (ps_profile);
        is_ps = SCE_FALSE;
    }
    else if (is_ps == SCE_FALSE)
    {
        /* NOTE: verifier l'ordre de desactivation!!! (Unbind & Disable) */
        cgGLDisableProfile (vs_profile);
        cgGLUnbindProgram (vs_profile);
        is_ps = -1;
    }
}
#endif

/* *** */

/** !! ** fonctions reservees aux shaders GLSL ** !! **/

SCE_CProgram* SCE_CCreateProgram (void)
{
    SCE_CProgram *prog = NULL;

    SCE_btstart ();
    prog = SCE_malloc (sizeof *prog);
    if (!prog)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return NULL;
    }

    prog->id = glCreateProgram ();
    /* et on teste pas la valeur de retour ?? */
    prog->compiled = SCE_FALSE;

    SCE_btend ();
    return prog;
}

void SCE_CDeleteProgram (SCE_CProgram *prog)
{
    SCE_btstart ();
    if (prog)
    {
        if (glIsProgram (prog->id))
            glDeleteProgram (prog->id);
        SCE_free (prog);
    }
    SCE_btend ();
}

int SCE_CSetProgramShader (SCE_CProgram *prog, SCE_CShaderGLSL *shader,
                           int attach)
{
    SCE_btstart ();
    if (attach)
        glAttachShader (prog->id, shader->id);
    else
        glDetachShader (prog->id, shader->id);

    /* il faut le recompiler pour appliquer les modifications! */
    prog->compiled = SCE_FALSE;
    SCE_btend ();
    return SCE_OK;
}

int SCE_CBuildProgram (SCE_CProgram *prog)
{
    int link_status = GL_TRUE;
    int loginfo_size = 0;
    char *loginfo = NULL;  /* chaine du log d'erreur */

    SCE_btstart ();
    /* lie le program a ses shaders 'attaches' */
    glLinkProgram (prog->id);

    /* recuperation du status du liage */
    glGetProgramiv (prog->id, GL_LINK_STATUS, &link_status);
    if (link_status != GL_TRUE)
    {
        /* erreur de 'linkage', recuperation du message d'erreur */

        Logger_Log (SCE_INVALID_OPERATION);

        glGetProgramiv (prog->id, GL_INFO_LOG_LENGTH, &loginfo_size);
        loginfo = SCE_malloc (loginfo_size + 1);
        if (!loginfo)
        {
            Logger_LogSrc ();
            SCE_btend ();
            return SCE_ERROR;
        }
        memset (loginfo, '\0', loginfo_size + 1);
        glGetProgramInfoLog (prog->id, loginfo_size, &loginfo_size, loginfo);

        Logger_LogMsg ("can't link program, reason : %s", loginfo);

        SCE_free (loginfo);
        SCE_btend ();
        return SCE_ERROR;
    }

    prog->compiled = SCE_TRUE;
    SCE_btend ();
    return SCE_OK;
}

void SCE_CUseProgram (SCE_CProgram *prog)
{
    if (prog)
        glUseProgram (prog->id);
    else
        glUseProgram (0);
}

/** ********************************************* **/


#if 0
void SCE_CDisableShaderGLSL (void)
{
    glUseProgram (0);
}

void SCE_CDisableShaderASM (int type)
{
    if (type == SCE_PIXEL_SHADER)
    {
        glBindProgramARB (GL_FRAGMENT_PROGRAM_ARB, 0);
        glDisable (GL_FRAGMENT_PROGRAM_ARB);
    }
    else
    {
        glBindProgramARB (GL_VERTEX_PROGRAM_ARB, 0);
        glDisable (GL_VERTEX_PROGRAM_ARB);
    }
}
#ifdef _SCE_USE_CG
void SCE_CDisableShaderCG (int type)
{
    if (type == SCE_PIXEL_SHADER)
    {
        cgGLDisableProfile (ps_profile);
        cgGLUnbindProgram (ps_profile);
    }
    else
    {
        cgGLDisableProfile (vs_profile);
        cgGLUnbindProgram (vs_profile);
    }
}
#endif
#endif


SCEint SCE_CGetProgramIndex (SCE_CProgram *prog, const char *name)
{
    return glGetUniformLocation (prog->id, name);
}

/* rien pour ASM, c'est gere au niveau de l'interface du moteur (p-e...) */

#ifdef _SCE_USE_CG
CGparameter SCE_CGetShaderCGIndex (SCE_CShaderCG *shader, const char *name)
{
    return cgGetNamedParameter (shader->id, name);
}
#endif





void SCE_CSetProgramParam (SCEint index, int val)
{
    glUniform1i (index, val);
}

void SCE_CSetProgramParamf (SCEint index, float val)
{
    glUniform1f (index, val);
}

void SCE_CSetProgramParamfv (SCEint index, size_t size, const float *val)
{
    switch (size)
    {
    case 1: glUniform1fv (index, 1, val); break;
    case 2: glUniform2fv (index, 1, val); break;
    case 3: glUniform3fv (index, 1, val); break;
    case 4: glUniform4fv (index, 1, val);
    }
}

void SCE_CSetProgramMatrix (SCEint index, size_t size, const float *mat)
{
    switch (size)
    {
    case 4: glUniformMatrix2fv (index, 1, SCE_FALSE, mat); break;
    case 9: glUniformMatrix3fv (index, 1, SCE_FALSE, mat); break;
    case 16: glUniformMatrix4fv (index, 1, SCE_FALSE, mat);
    }
}



void SCE_CSetShaderASMParamf (SCEuint index, float x, float y, float z, float w)
{
    if (last_target)
        glProgramLocalParameter4fARB (last_target, index, x, y, z, w);
}

void SCE_CSetShaderASMParamfv (SCEuint index, size_t size, const float *vars)
{
    if (last_target)
    {
        switch (size)
        {
        case 1:
            glProgramLocalParameter4fARB (last_target, index,
                                          vars[0], 0.0, 0.0, 1.0);
            break;
        case 2:
            glProgramLocalParameter4fARB (last_target, index,
                                          vars[0], vars[1], 0.0, 1.0);
            break;
        case 3:
            glProgramLocalParameter4fARB (last_target, index,
                                          vars[0], vars[1], vars[2], 1.0);
            break;
        case 4:
            glProgramLocalParameter4fvARB (last_target, index, vars);
        }
    }
}


#ifdef _SCE_USE_CG
void SCE_CSetShaderCGParam (CGparameter param, int val)
{
    cgGLSetParameter1f (param, (float)val);
}

void SCE_CSetShaderCGParamf (CGparameter param, float val)
{
    cgGLSetParameter1f (param, val);
}

void SCE_CSetShaderCGParamfv (CGparameter param, size_t size, const float *val)
{
    switch (size)
    {
    case 1: cgGLSetParameter1fv (param, val); break;
    case 2: cgGLSetParameter2fv (param, val); break;
    case 3: cgGLSetParameter3fv (param, val); break;
    case 4: cgGLSetParameter4fv (param, val); break;
    }
}

void SCE_CSetShaderCGMatrix (CGparameter param, const float *mat)
{
    cgGLSetMatrixParameterfc (param, mat);
}
#endif /* _SCE_USE_CG */
