/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 9 juillet 2007
   derniere modification : 09/07/2007 */


#include <SCE/SCEMinimal.h>

#include <SCE/core/SCECore.h>

#include <SCE/interface/SCEMesh.h>
#include <SCE/interface/SCEMaterial.h>

#include <SCE/interface/SCESkybox.h>


/* ajoute le 09/07/2007 */
/* revise le 20/10/2007 */
SCE_SSkybox* SCE_Skybox_Create (float l)
{
    float vertices[] =
    {
        -l, -l, -l,
         l, -l, -l,
         l,  l, -l,
        -l,  l, -l,
        
        -l, -l,  l,
         l, -l,  l,
         l,  l,  l,
        -l,  l,  l
    };

    unsigned char indices[] =
    {
        0, 1, 2, 3,
        4, 5, 6, 7,
        1, 2, 6, 5,
        0, 1, 5, 4,
        0, 3, 7, 4,
        2, 3, 7, 6
    };

    SCE_SSkybox *sky = SCE_malloc (sizeof *sky);
    if (!sky)
    {
        Logger_LogSrc ();
        return NULL;
    }

    sky->material = SCE_Material_Create ();
    if (!sky->material)
    {
        SCE_Skybox_Delete (sky);
        Logger_LogSrc ();
        return NULL;
    }

    sky->cube = SCE_Mesh_Create ();
    if (!sky->cube)
    {
        SCE_Skybox_Delete (sky);
        Logger_LogSrc ();
        return NULL;
    }

    /* creation du mesh */
    /* ajouts des donnees */
    if (SCE_Mesh_AddVerticesDup (sky->cube, 0, SCE_POSITION, SCE_FLOAT,
                                 3, 8, vertices) < 0)
    {
        SCE_Skybox_Delete (sky);
        Logger_LogSrc ();
        return NULL;
    }
    if (SCE_Mesh_SetIndicesDup (sky->cube, 0,
                                SCE_UNSIGNED_BYTE, 24, indices) < 0)
    {
        SCE_Skybox_Delete (sky);
        Logger_LogSrc ();
        return NULL;
    }
    /* construction */
    if (SCE_Mesh_Build (sky->cube) < 0)
    {
        SCE_Skybox_Delete (sky);
        Logger_LogSrc ();
        return NULL;
    }
    /* activation des tampons */
    SCE_Mesh_ActiveVB (sky->cube, 0, SCE_TRUE);
    SCE_Mesh_ActiveIB (sky->cube, SCE_TRUE);
    /* selection du mode de rendu */
    SCE_Mesh_SetRenderMode (sky->cube, SCE_QUADS);
    
    return sky;
}

/* ajoute le 09/07/2007 */
/* revise le 10/07/2007 */
void SCE_Skybox_Delete (SCE_SSkybox *sky)
{
    if (sky)
    {
        SCE_Mesh_Delete (sky->cube);
        SCE_Material_Delete (sky->material);
        SCE_free (sky);
    }
}


/* ajoute le 10/07/2007 */
/* revise le 20/10/2007 */
void SCE_Skybox_Render (SCE_SSkybox *sky)
{
    if(sky != NULL)
    {
        /* need glPushAttrib ... */
        /* desactivation du depth buffer */
        SCE_CSetState (GL_DEPTH_TEST, 0);
        SCE_Material_Use (sky->material);
        SCE_Mesh_Render (sky->cube);
        /*SCE_Material_Use (NULL);*/
    }
}
